"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
/**
 * Transforms an NDJSON stream to a stream of message objects
 */
class NdjsonToMessageStream extends stream_1.Transform {
    constructor(
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    fromObject) {
        super({ writableObjectMode: false, readableObjectMode: true });
        this.fromObject = fromObject;
    }
    _transform(chunk, encoding, callback) {
        if (this.buffer === undefined) {
            this.buffer = '';
        }
        this.buffer += Buffer.isBuffer(chunk) ? chunk.toString('utf-8') : chunk;
        const lines = this.buffer.split('\n');
        this.buffer = lines.pop();
        for (const line of lines) {
            if (line.trim().length > 0) {
                try {
                    const object = JSON.parse(line);
                    this.push(this.fromObject(object));
                }
                catch (err) {
                    return callback(new Error(`Not JSON: ${line}`));
                }
            }
        }
        callback();
    }
    _flush(callback) {
        if (this.buffer) {
            try {
                const object = JSON.parse(this.buffer);
                this.push(this.fromObject(object));
            }
            catch (err) {
                return callback(new Error(`Not JSONs: ${this.buffer}`));
            }
        }
        callback();
    }
}
exports.default = NdjsonToMessageStream;
//# sourceMappingURL=NdjsonToMessageStream.js.map